@@ -5,7 +5,7 @@ use strict;
 use warnings;
 no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = 0.08;
+our $VERSION = 0.09;
 
 my$self;
 
@@ -18,36 +18,35 @@ sub new{
 	my $class = ref($type)||$type;
 	$self = {
 		totalTransactions => undef,
-		minSupport => undef,
+		minSupport => 0,
 		minConfidence => undef,
+		minLift => undef,
+		minLeverage => undef,
+		minConviction => undef,
+		minCoverage => undef,
 		output => undef,
 		messages => undef,
 		itemsKeyDescription => undef,
 		itemsKeyTransactions => undef,
 		quantityPossibleRules => undef,
+		limitRules => undef,
 		frequentItemset => [],
 		recursively => 0,
 		associationRules => undef,
 		implications => undef,
 		itemsetSize => 2,
-		rule => 0
+		rule => 1
 	};
 	bless($self,$class);
 	return $self;
 }
 
 sub validate_data{
-	($self->{totalTransactions}=~/^\d+$/)
-		or die("Error: \$apriori->{totalTransactions} invalid value!\n");
-	($self->{minSupport}=~/^\d{1,3}\.?\d*$/
-	&&($self->{minSupport}>=0&&$self->{minSupport}<=100))
-		or die("Error: \$apriori->{minSupport} invalid value!\n");
-	($self->{minConfidence}=~/^\d{1,3}\.?\d*$/
-	&&($self->{minConfidence}>=0&&$self->{minConfidence}<=100))
-		or die("Error: \$apriori->{minConfidence} invalid value!\n");
-	($self->{itemsKeyDescription})
+	(defined $self->{totalTransactions})
+		or die("Error: \$apriori->{totalTransactions} is not defined!\n");
+	(defined $self->{itemsKeyDescription})
 		or die("Error: \$apriori->{itemsKeyDescription} is not defined!\n");
-	($self->{itemsKeyTransactions})
+	(defined $self->{itemsKeyTransactions})
 		or die("Error: \$apriori->{itemsKeyTransactions} is not defined!\n");
 }
 
@@ -124,18 +123,25 @@ for(my$antecedentSize=0;$antecedentSize<$#candidateItemset;$antecedentSize++){
 	my $supportConsequent = scalar(@transactions);
 	next if($supportConsequent == 0);
 	my $support = sprintf("%.2f",(($supportConsequent/$self->{totalTransactions})*100));
-	if($support >= $self->{minSupport}){
-		my $confidence = sprintf("%.2f",(($supportConsequent/$supportAntecedent)*100));
-		if($confidence >= $self->{minConfidence}){
-			$self->{rule}++;
-			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence];
-			$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
-			my@items=grep{!($_~~@newCandidateItemsetSizeOne)}@antecedent;
-			push @newCandidateItemsetSizeOne,@items;
-			@items=grep{!($_~~@newCandidateItemsetSizeOne)}@consequent;
-			push @newCandidateItemsetSizeOne,@items;			
-		}
-	}
+	next if($support < $self->{minSupport});
+	my $confidence = sprintf("%.2f",(($supportConsequent/$supportAntecedent)*100));
+	next if(defined $self->{minConfidence} && $confidence < $self->{minConfidence});
+	my $lift = sprintf("%.2f",($support/($supportAntecedent*$supportConsequent)));
+	next if(defined $self->{minLift} && $lift < $self->{minLift});
+	my $leverage = sprintf("%.2f",($support-($supportAntecedent*$supportConsequent)));
+	next if(defined $self->{minLeverage} && $leverage < $self->{minLeverage});	
+	my $conviction = sprintf("%.2f",((1-$supportAntecedent)/(1-$confidence)));
+	next if(defined $self->{minConviction} && $conviction < $self->{minConviction});
+	my $coverage = sprintf("%.2f",(($supportAntecedent/$self->{totalTransactions})*100));
+	next if(defined $self->{minCoverage} && $coverage < $self->{minCoverage});
+	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{rule} = ["{ @antecedent } => { @consequent }", $support, $confidence, $lift, $leverage, $conviction, $coverage];
+	$self->{associationRules}{$self->{rule}}{"R$self->{rule}"}{items} = [@antecedent, @consequent];
+	my@items=grep{!($_~~@newCandidateItemsetSizeOne)}@antecedent;
+	push @newCandidateItemsetSizeOne,@items;
+	@items=grep{!($_~~@newCandidateItemsetSizeOne)}@consequent;
+	push @newCandidateItemsetSizeOne,@items;
+	return if defined $self->{limitRules} && $self->{rule} == $self->{limitRules};
+	$self->{rule}++;
 }
 CODE
 		for(my$j=2;$j<=$itemsetSize;$j++){
@@ -161,7 +167,7 @@ CODE
 		}
 	}
 	if((scalar(@newCandidateItemsetSizeOne)>=($self->{itemsetSize}+1))&&$self->{recursively}){
-		$self->{itemsetSize}+=1;
+		$self->{itemsetSize}++;
 		$self->{associationRules} = undef;
 		$self->association_rules_itemset_size(\@newCandidateItemsetSizeOne);
 	}
@@ -235,17 +241,32 @@ sub file{
 		print "\nExporting to file \"output_itemset_size_$self->{itemsetSize}.txt\"... ";
 	}
 	open(FILE,">output_itemset_size_$self->{itemsetSize}.txt");
-	print FILE "Rules\tSupport %\tConfidence %\n";
+	print FILE "Rules\tSupport %";
+	print FILE "\tConfidence %" if defined $self->{minConfidence};
+	print FILE "\tLift" if defined $self->{minLift};
+	print FILE "\tLeverage" if defined $self->{minLeverage};
+	print FILE "\tConviction" if defined $self->{minConviction};
+	print FILE "\tCoverage %" if defined $self->{minCoverage};
+	print FILE "\n";
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
-		${$self->{associationRules}{$rule}{"R$rule"}{rule}}[1]=~s/\./,/;
-		${$self->{associationRules}{$rule}{"R$rule"}{rule}}[2]=~s/\./,/;
-		print FILE "R$rule\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1]\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2]\n";
+		foreach(@{$self->{associationRules}{$rule}{"R$rule"}{rule}}){s/\./,/;}
+		print FILE "R$rule\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1]";
+		print FILE "\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2]" if defined $self->{minConfidence};
+		print FILE "\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[3]" if defined $self->{minLift};
+		print FILE "\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[4]" if defined $self->{minLeverage};
+		print FILE "\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[5]" if defined $self->{minConviction};
+		print FILE "\t${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[6]" if defined $self->{minCoverage};
+		print FILE "\n";
 	}
 	print FILE "\n";
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
 		print FILE "Rule R$rule: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]\n";
 		print FILE "Support: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %\n";
-		print FILE "Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %\n";
+		print FILE "Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %\n" if defined $self->{minConfidence};
+		print FILE "Lift: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[3]\n" if defined $self->{minLift};
+		print FILE "Leverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[4]\n" if defined $self->{minLeverage};
+		print FILE "Conviction: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[5]\n" if defined $self->{minConviction};
+		print FILE "Coverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[6] %\n" if defined $self->{minCoverage};
 		print FILE "Items:\n";
 		foreach my$item(@{$self->{associationRules}{$rule}{"R$rule"}{items}}){
 			print FILE "$item $self->{itemsKeyDescription}{$item}\n";
@@ -269,37 +290,106 @@ sub excel{
 		or die("\nError: \"output_itemset_size_$self->{itemsetSize}.xlsx\" $!");
 	my $worksheet = $workbook->add_worksheet();
 	my $bold = $workbook->add_format(bold => 1);
-	my $headings = ['Rules', 'Support %', 'Confidence %'];
-	my(@rules,@support,@confidence);	
+	my $headings = ['Rules', 'Support %'];
+	push @{$headings},'Confidence %' if defined $self->{minConfidence};
+	push @{$headings},'Lift' if defined $self->{minLift};
+	push @{$headings},'Leverage' if defined $self->{minLeverage};
+	push @{$headings},'Conviction' if defined $self->{minConviction};
+	push @{$headings},'Coverage' if defined $self->{minCoverage};
+	my(@rules,@support,@confidence,@lift,@leverage,@conviction,@coverage);	
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
 		push @rules,"R$rule";
 		push @support,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[1];
-		push @confidence,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[2];
+		push @confidence,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[2] if defined $self->{minConfidence};
+		push @lift,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[3] if defined $self->{minLift};
+		push @leverage,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[4] if defined $self->{minLeverage};
+		push @conviction,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[5] if defined $self->{minConviction};
+		push @coverage,${$self->{associationRules}{$rule}{"R$rule"}{rule}}[6] if defined $self->{minCoverage};
 	}
 	my$line=(scalar(@rules)+1);
-	my@data=(\@rules,\@support,\@confidence);
+	my@data=(\@rules,\@support,);
+	push @data,\@confidence if defined $self->{minConfidence};
+	push @data,\@lift if defined $self->{minLift};
+	push @data,\@leverage if defined $self->{minLeverage};
+	push @data,\@conviction if defined $self->{minConviction};
+	push @data,\@coverage if defined $self->{minCoverage};
 	$worksheet->write('A1', $headings, $bold);
 	$worksheet->write('A2', \@data);
 	my$chart=$workbook->add_chart(type => 'column', embedded => 1);
+	my@columns=('B'..'G');
+	my$i=0;
 	$chart->add_series(
 		name       => 'Support %',
 		categories => '=Sheet1!$A$2:$A$'.$line,
-		values     => '=Sheet1!$B$2:$B$'.$line,
+		values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
 	);
-	$chart->add_series(
-		name       => 'Confidence %',
-		categories => '=Sheet1!$A$2:$A$'.$line,
-		values     => '=Sheet1!$C$2:$C$'.$line,
-	);
-	$worksheet->insert_chart('H2', $chart);
+	if(defined $self->{minConfidence}){
+		$i++;
+		$chart->add_series(
+			name       => 'Confidence %',
+			categories => '=Sheet1!$A$2:$A$'.$line,
+			values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
+		);
+	}
+	if(defined $self->{minLift}){
+		$i++;
+		$chart->add_series(
+			name       => 'Lift',
+			categories => '=Sheet1!$A$2:$A$'.$line,
+			values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
+		);
+	}
+	if(defined $self->{minLeverage}){
+		$i++;
+		$chart->add_series(
+			name       => 'Leverage',
+			categories => '=Sheet1!$A$2:$A$'.$line,
+			values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
+		);
+	}
+	if(defined $self->{minConviction}){
+		$i++;
+		$chart->add_series(
+			name       => 'Conviction',
+			categories => '=Sheet1!$A$2:$A$'.$line,
+			values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
+		);
+	}
+	if(defined $self->{minCoverage}){
+		$i++;
+		$chart->add_series(
+			name       => 'Coverage %',
+			categories => '=Sheet1!$A$2:$A$'.$line,
+			values     => '=Sheet1!$'.$columns[$i].'$2:$'.$columns[$i].'$'.$line,
+		);
+	}
+	$worksheet->insert_chart('I2', $chart);
 	$line+=2;
 	foreach my$rule(sort{$a<=>$b}keys(%{$self->{associationRules}})){
 		$worksheet->write("A$line","Rule R$rule: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[0]");
 		$line++;
 		$worksheet->write("A$line","Support: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[1] %");
 		$line++;
-		$worksheet->write("A$line","Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %");
-		$line++;
+		if(defined $self->{minConfidence}){
+			$worksheet->write("A$line","Confidence: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[2] %");
+			$line++;
+		}
+		if(defined $self->{minLift}){
+			$worksheet->write("A$line","Lift: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[3]");
+			$line++;
+		}
+		if(defined $self->{minLeverage}){
+			$worksheet->write("A$line","Leverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[4]");
+			$line++;
+		}
+		if(defined $self->{minConviction}){
+			$worksheet->write("A$line","Conviction: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[5]");
+			$line++;
+		}
+		if(defined $self->{minCoverage}){
+			$worksheet->write("A$line","Coverage: ${$self->{associationRules}{$rule}{\"R$rule\"}{rule}}[6]");
+			$line++;
+		}
 		$worksheet->write("A$line","Items:");
 		$line++;
 		foreach my$item(@{$self->{associationRules}{$rule}{"R$rule"}{items}}){
@@ -347,9 +437,19 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{totalTransactions}=10; # The total number of transactions
 
-	$apriori->{minSupport}=1.55; # The minimum support (percent)
+	$apriori->{minSupport}=1.55; # The minimum support(percent)
 
-	$apriori->{minConfidence}=1.55; # The minimum confidence (percent)
+	$apriori->{minConfidence}=1.55; # The minimum confidence(percent, optional)
+
+	$apriori->{minLift}=0; # The minimum lift(optional)
+
+	$apriori->{minLeverage}=0; # The minimum leverage(optional)
+
+	$apriori->{minConviction}=0; # The minimum conviction(optional)
+
+	$apriori->{minCoverage}=0; # The minimum coverage(percent, optional)
+
+	$apriori->{limitRules}=10; # The limit of rules(optional)
 
 	$apriori->{output}=1; # The output type (1 - Export to text file delimited by tab; 2 - Export to excel file with chart)(optional)
 
@@ -385,44 +485,56 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	#output file "output_itemset_size_2.txt"
 
-	Rules	Support %	Confidence %
-	R1	70,00	87,50
-	R2	70,00	87,50
-	R3	70,00	77,78
-	R4	80,00	88,89
-	R5	70,00	77,78
-	R6	80,00	88,89
+	Rules	Support %	Confidence %	Lift	Leverage	Conviction	Coverage %
+	R1	70,00	77,78	1,11	7,00	0,10	90,00
+	R2	80,00	88,89	1,11	8,00	0,09	90,00
+	R3	70,00	87,50	1,25	14,00	0,08	80,00
+	R4	70,00	87,50	1,25	14,00	0,08	80,00
+	R5	80,00	88,89	1,11	8,00	0,09	90,00
+	R6	70,00	77,78	1,11	7,00	0,10	90,00
 
-	Rule R1: { 101 } => { 103 }
+	Rule R1: { 103 } => { 101 }
 	Support: 70,00 %
-	Confidence: 87,50 %
+	Confidence: 77,78 %
+	Lift: 1,11
+	Leverage: 7,00
+	Conviction: 0,10
+	Coverage: 90,00 %
 	Items:
-	101 MILK
 	103 CEREAL
+	101 MILK
 
 	to be continued...
 
 	#output file "output_itemset_size_3.txt"
 
-	Rules	Support %	Confidence %
-	R7	60,00	75,00
-	R8	60,00	85,71
-	R9	60,00	85,71
-	R10	60,00	66,67
-	R11	60,00	75,00
-	R12	60,00	66,67
+	Rules	Support %	Confidence %	Lift	Leverage	Conviction	Coverage %
+	R7	60,00	66,67	1,11	6,00	0,12	90,00
+	R8	60,00	85,71	1,43	18,00	0,07	70,00
+	R9	60,00	75,00	1,25	12,00	0,09	80,00
+	R10	60,00	75,00	1,25	12,00	0,09	80,00
+	R11	60,00	85,71	1,43	18,00	0,07	70,00
+	R12	60,00	66,67	1,11	6,00	0,12	90,00
 
-	Rule R7: { 101 } => { 102, 103 }
+	Rule R7: { 103 } => { 101, 102 }
 	Support: 60,00 %
-	Confidence: 75,00 %
+	Confidence: 66,67 %
+	Lift: 1,11
+	Leverage: 6,00
+	Conviction: 0,12
+	Coverage: 90,00 %
 	Items:
+	103 CEREAL
 	101 MILK
 	102 BREAD
-	103 CEREAL
 
 	Rule R8: { 101, 103 } => { 102 }
 	Support: 60,00 %
 	Confidence: 85,71 %
+	Lift: 1,43
+	Leverage: 18,00
+	Conviction: 0,07
+	Coverage: 70,00 %
 	Items:
 	101 MILK
 	103 CEREAL
@@ -505,6 +617,16 @@ Data::Mining::Apriori - Perl extension for implement the apriori algorithm of da
 
 	$apriori->{minConfidence}=1.55;
 
+	$apriori->{minLift}=0;
+
+	$apriori->{minLeverage}=0;
+
+	$apriori->{minConviction}=0;
+
+	$apriori->{minCoverage}=0;
+
+	$apriori->{limitRules}=10;
+
 	$apriori->{output}=1;
 
 	$apriori->{messages}=1;
@@ -557,11 +679,31 @@ The total number of transactions.
 
 =head2 minSupport
 
-The minimum support (percent).
+The minimum support(percent).
 
 =head2 minConfidence
 
-The minimum confidence (percent).
+The minimum confidence(percent, optional).
+
+=head2 minLift
+
+The minimum lift(optional).
+
+=head2 minLeverage
+
+The minimum leverage(optional).
+
+=head2 minConviction
+
+The minimum conviction(optional).
+
+=head2 minCoverage
+
+The minimum coverage(percent, optional).
+
+=head2 limitRules
+
+The limit of rules(optional).
 
 =head2 output
 
@@ -569,13 +711,9 @@ The output type (optional):
 
 =over 4
 
-=item 1-
+=item * 1 - Text file delimited by tab;
 
-Text file delimited by tab;
-
-=item 2-
-
-Excel file with chart.
+=item * 2 - Excel file with chart.
 
 =back
 
@@ -605,23 +743,27 @@ A value boolean to generate association rules until no set of items meets the mi
 
 =head2 associationRules
 
-a data structure to store the name of the rule, key items, implication, support and confidence.
+A data structure to store the name of the rule, key items, implication, support, confidence, lift, leverage, conviction and coverage.
 
 	$self->{associationRules} = {
-								  '1' => {
-										   'R1' => {
-													 'items' => [
-																  '101',
-																  '103'
-																],
-													 'rule' => [
-																 '{ 101 } => { 103 }',
-																 '70.00',
-																 '87.50'
-															   ]
-												   }
-										 },
-									# to be continued...
+                                    '1' => {
+                                                'R1' => {
+                                                            'items' => [
+                                                                         '103',
+                                                                         '101'
+                                                                       ],
+                                                            'rule' => [
+                                                                        '{ 103 } => { 101 }',
+                                                                        '70,00',
+                                                                        '77,78',
+                                                                        '1,11',
+                                                                        '7,00',
+                                                                        '0,10',
+                                                                        '90,00'
+                                                                      ]
+                                                        }
+                                            },
+                                            # to be continued...
 
 =head1 METHODS
 
@@ -631,7 +773,7 @@ Creates a new instance of Data::Mining::Apriori.
 
 =head2 generate_rules
 
-Generate association rules until no set of items meets the minimum support or minimum confidence.
+Generate association rules until no set of items meets the minimum support, confidence, lift, leverage, conviction and coverage.
 
 =head2 association_rules_itemset_size
 
